# Making functions so that can start at same A/a (H/h in the 
# manuscript) frequencies each time

rm(list=ls())
source("Recombination Dec 2011 Dryad.R")
source("parameters Dec 2011 Dryad.R")
source("Setting up Pop'n Dec 2011 Dryad.R")

# The following function is designed to create a
# burn-in of 10 cycles between hosts and parasites
# The final values of A/a in the population (referred
# to as H/h in the manuscript) are used to initiate 
# multiple runs afterwards

obtain.A <- function(nh, xp, mu.h, mu.p, alpha.h, alpha.p, beta, ra, rb, rc, R, num) {

	output.h <- matrix(NA, when.intro, length(nh))
	output.p <- matrix(NA, when.intro)
	
	i <- 0
	
	# The hosts and parasites go through cycles of selection
	# mutation and recombination
	repeat {
		i <- i + 1
		
		# Population is started based on the function
		# "Setting up Pop'n Dec 2011 Dryad.R" where only
		# two of eight host genotypes are present: Abm and
		# abm (which is Hbm and hbm in the manuscript)
		xh <- nh/num
		
		# Host fitness function where each genotype is given a 
		# fitness based on its probability of encounter with
		# a matching parasite
		wh <- (1 + alpha.h * (gtypes$A * xp + (1 - gtypes$A) * (1 - xp))) * (1 + (gtypes$B) * beta)
		xhtemp <- next.gen.h(xh, wh)
		
		# Parasite fitness function similarly based on the 
		# probability of the parasite allele encountering a 
		# matching host
		ph <- sum(gtypes$A * xh)
		wp <- 1 + alpha.p * c(ph, 1-ph)
		# The parasites then produce an effectively infinite 
		# number of parasite propagules
		xptemp <- next.gen.p(xp, wp)

		xh <- xhtemp
		xp <- xptemp
		
		# Both hosts and parasites go through mutation
		xh <- mutation.h(mu.h, xh) 
		xp <- mutation.p(mu.p, xp)
		
		# Only the host then goes through recombination
		# according to the recombination function in
		# "Recombination Dec 2011 Dryad.R" however at this 
		# point only the A/a (H/h) alleles are present in 
		# the population
		xh <- recombination(c(xh), ra, rb, rc, R)
		
		nh <- matrix(rmultinom(1, size=num, prob=xh), ncol=8)
		
		output.h[i,] <- nh
		output.p[i] <- xp
		
		if(i==when.intro)
			break()
	}
	
	list(h=output.h[nrow(output.h),], p=output.p[nrow(output.p),], gen=i)
}


# This function then takes the burn-ins and introduces the B
# allele, while also bringing the M allele into the population
# at a frequency of 50% among all of the hosts and the population
# then goes through cycles until the B allele is either fixed or
# lost
over.time <- function(nh.new, xp.new, mu.h, mu.p, alpha.h, alpha.p, beta, ra, rb, rc, R, num) {

	nh <- nh.new
	xp <- xp.new
	
	# B and M are brought into the population of hosts
	nh <- intro.b(nh) 
	output.h <- matrix(nh, nrow=8)
	output.p <- matrix(xp)
	i <- 0
	
	repeat {
		i <- i + 1
		
		nh.old <- nh
		xp.old <- xp
		
		xh <- nh/num
		
		# Fitness, mutation and selection functions are the same
		# as above.
		wh <- (1 + alpha.h * (gtypes$A * xp + (1 - gtypes$A) * (1 - xp))) * 			(1 + (gtypes$B) * beta)
		xhtemp <- next.gen.h(xh, wh)
		
		ph <- sum(gtypes$A * xh)
		wp <- 1 + alpha.p * c(ph, 1-ph)
		xptemp <- next.gen.p(xp, wp)
		
		xh <- xhtemp
		xp <- xptemp

		xh <- mutation.h(mu.h, xh) 
		xp <- mutation.p(mu.p, xp)
		
		xh <- recombination(c(xh), ra, rb, rc, R)
		
		nh <- matrix(rmultinom(1, size=num, prob=xh), nrow=8)

		output.h <- cbind(output.h, nh)
		output.p <- cbind(output.p, xp)
			
		total.b <- sum(c(nh[2], nh[4], nh[6], nh[8]))
		total.B <- sum(c(nh[1], nh[3], nh[5], nh[7]))
		
		# The populations are tracked through cycles until
		# the B allele is either fixed or lost
		if(total.b==0)
			break()
		if(total.B==0)
			break()		
	}	
	list(h=t(output.h), p=t(output.p), gen=i)	
}


# In order to do multiple runs with the same burn-in ratios
# of the A/a (H/h) alleles, introducing B and tracking whether 
# it is fixed or lost in the population the following function 
# was written such that there are nruns of the 'over.time' 
# function using the same burn-in allele frequencies

multiple.runs <- function(nruns, nh.new, xp.new, f.A, f.B, f.M, num, mu.h, mu.p, alpha.p, alpha.h, beta, ra, rb, rc, R, gtypes, initialize){
	product.freq <- matrix(NA, nrow=nruns, ncol=8)
	product.M <- rep(NA, nruns)
	product.gen <- rep(NA, nruns)
	
	for(i in seq_len(nruns)){		
		poph.matrix <- matrix(over.time(nh.new,xp.new,mu.h,mu.p,alpha.h,alpha.p,beta,ra,rb,rc,R,num)$h, ncol=8)
		
		final.f <- poph.matrix[nrow(poph.matrix),]
		final.M <- sum(final.f[1:4])
		
		product.freq[i,] <- final.f
		product.M[i] <- final.M
		product.gen[i] <- nrow(poph.matrix)
	}
	matrix(c(product.freq, product.M, product.gen), ncol=10)
}


